package org.daligo.client;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.message.BasicNameValuePair;
import org.daligo.type.Course;
import org.daligo.type.Message;
import org.daligo.type.Milestone;

public class CenterClient extends RestClient {
	protected static final String CENTER_BASE_URL = "http://daligo.kuls.net/";

	public void getMessages() {
		this.get(CENTER_BASE_URL + "messages.xml");
	}

	public void getMessage(int id) {
		this.get(CENTER_BASE_URL + "messages/" + id + "/.xml");
	}

	public void createMessage(final Message newMessage) {
		HttpResponse httpResonse = this.post(CENTER_BASE_URL + "messages.xml",
				new HttpParameterMapper() {
					public List<NameValuePair> mapParameter() {
						List<NameValuePair> result = new ArrayList<NameValuePair>();
						result.add(new BasicNameValuePair("message[user_id]",
								newMessage.getUserId().toString()));
						result.add(new BasicNameValuePair("message[body]",
								newMessage.getBody()));
						return result;
					}
				});
	}

	public void deleteMessage(int id) {
		this.delete(CENTER_BASE_URL + "messages" + id + "/.xml");
	}

	public void createCourse(final Course newCourse) {
		HttpResponse httpResonse = this.post(CENTER_BASE_URL + "courses.xml", new HttpParameterMapper() {
			public List<NameValuePair> mapParameter() {
				List<NameValuePair> result = new ArrayList<NameValuePair>();
				result.add(new BasicNameValuePair("course[user_id]", "2"));
				result.add(new BasicNameValuePair("course[name]", newCourse
						.getName()));
				return result;
			}
		});		
		
		final Integer courseId = null; // TODO: httpResonse 에서 새로 생성된 Course ID 를 가져와 저장한다.

		for (int i = 0; i < newCourse.getMilestones().length; i++) {
			final Milestone milestone = newCourse.getMilestones()[i];
			this.post(CENTER_BASE_URL + "milestones",
					new HttpParameterMapper() {
						public List<NameValuePair> mapParameter() {
							List<NameValuePair> result = new ArrayList<NameValuePair>();
							result.add(new BasicNameValuePair("milestone[course_id]", courseId.toString()));
							result.add(new BasicNameValuePair("milestone[latitude]",
									milestone.getLatitude().toString()));
							result.add(new BasicNameValuePair("milestone[longitude]",
									milestone.getLongitude().toString()));
							return result;
						}
					});
		}
	}

	public void updateMessage(final Message updatedMessage) {
		this.put(CENTER_BASE_URL + "messages/"
				+ updatedMessage.getId().toString() + "/.xml",
				new HttpParameterMapper() {
					public List<NameValuePair> mapParameter() {
						List<NameValuePair> result = new ArrayList<NameValuePair>();
						result.add(new BasicNameValuePair("message[id]",
								updatedMessage.getId().toString()));
						result.add(new BasicNameValuePair("message[user_id]",
								updatedMessage.getUserId().toString()));
						result.add(new BasicNameValuePair("message[body]",
								updatedMessage.getBody()));
						result.add(new BasicNameValuePair(
								"message[created_at]", new Date().toString()));
						return result;
					}
				});

	}
}
