package org.daligo.domain;

import org.daligo.type.UserProfile;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteException;
import android.database.sqlite.SQLiteOpenHelper;
import android.database.sqlite.SQLiteDatabase.CursorFactory;
import android.util.Log;

public class ProfileDBAdapter {
	private static final String DATABASE_NAME = "daligo.db";
	private static final String DATABASE_TABLE = "profile";
	private static final int DATABASE_VERSION = 1;

	private SQLiteDatabase db;
	private final Context context;

	private profileDBOpenHelper dbHelper;

	public static final String KEY_ID = "_id";
	public static final int ID_COLUMN = 0;

	public static final String KEY_NAME = "name";
	public static final int NAME_COLUMN = 1;

	public static final String KEY_AGE = "age";
	public static final int AGE_COLUMN = 2;

	public static final String KEY_GENDER = "gender";
	public static final int GENDER_COLUMN = 3;

	public static final String KEY_HEIGHT = "height";
	public static final int HEIGHT_COLUMN = 4;

	public static final String KEY_WEIGHT = "weight";
	public static final int WEIGHT_COLUMN = 5;

	public ProfileDBAdapter(Context _context) {
		this.context = _context;
		dbHelper = new profileDBOpenHelper(context, DATABASE_NAME, null,
				DATABASE_VERSION);
	}

	private static class profileDBOpenHelper extends SQLiteOpenHelper {

		public profileDBOpenHelper(Context context, String name,
				CursorFactory factory, int version) {
			super(context, name, factory, version);
			// TODO Auto-generated constructor stub
		}

		// 새로운 데이터베이스를 만드는 sql문
		private static final String DATABASE_CREATE = "create table if not exists "
				+ DATABASE_TABLE
				+ " ("
				+ KEY_ID
				+ " integer primary key autoincrement, "
				+ KEY_NAME
				+ " text not null, "
				+ KEY_AGE
				+ " integer, "
				+ KEY_GENDER
				+ " integer, "
				+ KEY_HEIGHT
				+ " integer, "
				+ KEY_WEIGHT
				+ " integer); ";

		@Override
		public void onCreate(SQLiteDatabase _db) {
			_db.execSQL(DATABASE_CREATE);
		}

		@Override
		public void onUpgrade(SQLiteDatabase _db, int _oldVersion,
				int _newVersion) {
			Log.w("ProfileDBAdapter", "Upgrading from version " + _oldVersion
					+ " to " + _newVersion
					+ ", which will destroy all old data");
			// 기존 테이블을 버린다.
			_db.execSQL("DROP TABLE IF EXISTS " + DATABASE_TABLE);
			// 새것을 만든다
			onCreate(_db);
		}
	}

	public void close() {
		db.close();
	}

	public void open() throws SQLiteException {
		try {
			db = dbHelper.getWritableDatabase();
		} catch (SQLiteException ex) {
			db = dbHelper.getReadableDatabase();
		}
	}

	// 새 프로플을 삽입한다
	public long insertProfile(UserProfile _profile) {
		ContentValues profileValues = new ContentValues();

		profileValues.put(KEY_NAME, _profile.getName());
		profileValues.put(KEY_AGE, _profile.getAge());
		profileValues.put(KEY_GENDER, _profile.getGender());
		profileValues.put(KEY_HEIGHT, _profile.getHeight());
		profileValues.put(KEY_WEIGHT, _profile.getWeight());

		// 이 행을 삽입한다
		return db.insert(DATABASE_TABLE, null, profileValues);
	}

	// 프로필을 제거한다.
	public boolean removeProfle(long _rowIndex) {
		return db.delete(DATABASE_TABLE, KEY_ID + "=" + _rowIndex, null) > 0;
	}

	// 프로필을 업데이트 한다.
	public boolean updateProfile(long _rowIndex, UserProfile _profile) {
		ContentValues profileValues = new ContentValues();

		profileValues.put(KEY_NAME, _profile.getName());
		profileValues.put(KEY_AGE, _profile.getAge());
		profileValues.put(KEY_GENDER, _profile.getGender());
		profileValues.put(KEY_HEIGHT, _profile.getHeight());
		profileValues.put(KEY_WEIGHT, _profile.getWeight());

		return db.update(DATABASE_TABLE, profileValues, KEY_ID + "="
				+ _rowIndex, null) > 0;
	}

	public Cursor getAllProfileCursor() {
		return db.query(DATABASE_TABLE, new String[] { KEY_ID, KEY_NAME,
				KEY_AGE, KEY_GENDER, KEY_HEIGHT, KEY_WEIGHT }, null, null,
				null, null, null);

	}

	public Cursor setCursorProfile(long _rowIndex) throws SQLException {
		Cursor result = db.query(true, DATABASE_TABLE, new String[] { KEY_ID,
				KEY_NAME, KEY_AGE, KEY_GENDER, KEY_HEIGHT, KEY_WEIGHT }, KEY_ID
				+ "=" + _rowIndex, null, null, null, null, null);
		if ((result.getCount() == 0) || !result.moveToFirst()) {
			throw new SQLException("No to do items found for row: " + _rowIndex);
		}
		return result;
	}

	public UserProfile getUserProfile(long _rowIndex) throws SQLException {
		Cursor cursor = db.query(true, DATABASE_TABLE, new String[] { KEY_ID,
				KEY_NAME, KEY_AGE, KEY_GENDER, KEY_HEIGHT, KEY_WEIGHT }, KEY_ID
				+ "=" + _rowIndex, null, null, null, null, null);
		if ((cursor.getCount() == 0) || !cursor.moveToFirst()) {
			throw new SQLException("No to do items found for row: " + _rowIndex);
		}

		String name;
		int age;
		int gender;
		int height;
		int weight;

		name = cursor.getString(NAME_COLUMN);
		age = cursor.getInt(AGE_COLUMN);
		gender = cursor.getInt(GENDER_COLUMN);
		height = cursor.getInt(HEIGHT_COLUMN);
		weight = cursor.getInt(WEIGHT_COLUMN);

		UserProfile result = new UserProfile(name, age, gender, height, weight);
		return result;
	}
}
